// some needed headers
#include "StdH.h"
#include <windows.h>
#include "Sources/gl/gl.h"
#include "Sources/gl/glu.h"

// some stuff from the engine (no, we dont want to include everything)
#define ENGINE_API __declspec(dllimport)
typedef long int INDEX;
//ENGINE_API char * Translate(char *str, INDEX iOffset=0);
#define TRANS(str) Translate("ETRS" str, 4)
ENGINE_API extern void CPrintF(const char *strFormat, ...);

/* class __declspec(dllimport) CNetworkLibrary {
public:
BOOL IsPaused(void);
}; */
ENGINE_API extern CNetworkLibrary *_pNetwork;

// time in seconds
inline double GetAccurateTime(void) {
__int64 iCurrentTime;
__int64 iFrequency;
QueryPerformanceFrequency((LARGE_INTEGER*)&iFrequency);
QueryPerformanceCounter((LARGE_INTEGER*)& iCurrentTime);
return (double)iCurrentTime/(double)iFrequency;
}

// fps must be controlled to make the effect look same on faster computers too
#define BLUR_FPS_MAX 40
// this controls how many frames backwards are drawn (eg. the length of trails)
#define BLUR_TEXTURE_COUNT 5
static int unsigned _iBlurTexture[BLUR_TEXTURE_COUNT+1];
static int _iBlurDrawOnTexture=0;
static int _iBlurWidth=512;
static int _iBlurHeight=512;
static double _fBlurLastTime=0;


static int _iInitResolutionX=0;
static int _iInitResolutionY=0;
static bool _bMotionBlurLoaded=false;

void DestroyMotionBlur(void);

void InitMotionBlur(void) {

// get viewport dimensions
int iViewPort[4];
glGetIntegerv(GL_VIEWPORT,iViewPort);
int iScreenWidth=iViewPort[2];
int iScreenHeight=iViewPort[3];


// already loaded but resolution was changed
if(_bMotionBlurLoaded && (_iInitResolutionX!=iScreenWidth || _iInitResolutionY!=iScreenHeight)) {
// destroy
DestroyMotionBlur();
// reload
_bMotionBlurLoaded=false;
_iInitResolutionX=iScreenWidth;
_iInitResolutionY=iScreenHeight;
}

// already initialized - do nothing
if(_bMotionBlurLoaded) return;


// create textures
glEnable(GL_TEXTURE_2D);
_iBlurDrawOnTexture=0;
unsigned char* pPixels = new unsigned char[_iBlurWidth*_iBlurHeight*3];
memset(pPixels, 0, _iBlurWidth*_iBlurHeight*3);

glGenTextures(BLUR_TEXTURE_COUNT+1,_iBlurTexture);
for(int i=0;i<BLUR_TEXTURE_COUNT+1;i++) {
glBindTexture(GL_TEXTURE_2D, _iBlurTexture[i]);
glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB8, _iBlurWidth, _iBlurHeight, 0, GL_RGB8, GL_UNSIGNED_BYTE, pPixels);
}

delete[] pPixels;
_bMotionBlurLoaded=true;
}

void DestroyMotionBlur(void) {
_bMotionBlurLoaded=false;
glDeleteTextures(BLUR_TEXTURE_COUNT+1,_iBlurTexture);
}

int GetMotionBlurWidth(void) {
return _iBlurWidth;
}

int GetMotionBlurHeight(void) {
return _iBlurHeight;
}

void CaptureScreenForMotionBlur(void) {

// store the current projection matrix
glMatrixMode(GL_PROJECTION);
glPushMatrix();
glLoadIdentity();

// and set a new orthogonal one
gluOrtho2D(0,_iBlurWidth,_iBlurHeight,0);

// store the current modelview matrix
glMatrixMode(GL_MODELVIEW);
glPushMatrix();
glLoadIdentity();

// set some settings
glDisable(GL_DEPTH_TEST);
glDisable(GL_TEXTURE_2D);
glEnable(GL_TEXTURE_2D);
glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

// copy current screen
glEnable(GL_TEXTURE_2D);
glBindTexture(GL_TEXTURE_2D, _iBlurTexture[_iBlurDrawOnTexture]);
glCopyTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, 0, 0, _iBlurWidth, _iBlurHeight, 0);

int iTarget=_iBlurDrawOnTexture;

// switch rendering target texture
if(GetAccurateTime()-_fBlurLastTime>=(1/(float)BLUR_FPS_MAX)) {
if(!_pNetwork->IsPaused()) {
_iBlurDrawOnTexture++;
if(_iBlurDrawOnTexture>=BLUR_TEXTURE_COUNT) _iBlurDrawOnTexture=0;
_fBlurLastTime=GetAccurateTime();
}
}

// render from 2nd newest to oldest
int i=iTarget-1;
if(i<0) i+=BLUR_TEXTURE_COUNT;
float fAlpha=1;
while(i!=iTarget) {
glBindTexture(GL_TEXTURE_2D, _iBlurTexture[i]);

glColor4f(1,1,1,fAlpha);
glBegin(GL_QUADS);
glTexCoord2i( 0, 1);
glVertex3i(0, 0,  0);

glTexCoord2i( 1, 1);
glVertex3i(_iBlurWidth, 0,  0);

glTexCoord2i( 1, 0);
glVertex3i(_iBlurWidth, _iBlurHeight,  0);

glTexCoord2i( 0, 0);
glVertex3i(0, _iBlurHeight,  0);
glEnd();

fAlpha*=0.5f;
i--;
if(i<0) i+=BLUR_TEXTURE_COUNT;
}

// copy current view to temp texture
glBindTexture(GL_TEXTURE_2D, _iBlurTexture[BLUR_TEXTURE_COUNT]);
glCopyTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, 0, 0, _iBlurWidth, _iBlurHeight, 0);


// restore settings
glColor4f(1,1,1,1);
glEnable(GL_DEPTH_TEST);
// restore the old projection matrix
glMatrixMode(GL_PROJECTION);
glPopMatrix();
// restore the old model view matrix
glMatrixMode(GL_MODELVIEW);
glPopMatrix();

}

void RenderMotionBlur(float fEffectStrength, int iScreenWidth, int iScreenHeight) {

// store the current projection matrix
glMatrixMode(GL_PROJECTION);
glPushMatrix();
glLoadIdentity();

// and set a new orthogonal one
gluOrtho2D(0,iScreenWidth,iScreenHeight,0);

// store the current modelview matrix
glMatrixMode(GL_MODELVIEW);
glPushMatrix();
glLoadIdentity();

// set some settings
glDisable(GL_DEPTH_TEST);
glDisable(GL_TEXTURE_2D);
glEnable(GL_TEXTURE_2D);
glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

// draw temp texture
glBindTexture(GL_TEXTURE_2D, _iBlurTexture[BLUR_TEXTURE_COUNT]);

glColor4f(1,1,1,fEffectStrength);
glBegin(GL_QUADS);
glTexCoord2i( 0, 1);
glVertex3i(0, 0,  0);

glTexCoord2i( 1, 1);
glVertex3i(iScreenWidth, 0,  0);

glTexCoord2i( 1, 0);
glVertex3i(iScreenWidth, iScreenHeight,  0);

glTexCoord2i( 0, 0);
glVertex3i(0, iScreenHeight,  0);
glEnd();


// restore settings
glColor4f(1,1,1,1);
//glDisable(GL_BLEND);
glEnable(GL_DEPTH_TEST);
// restore the old projection matrix
glMatrixMode(GL_PROJECTION);
glPopMatrix();
// restore the old model view matrix
glMatrixMode(GL_MODELVIEW);
glPopMatrix();
}